// File:	TMACRO.H
// Author:	Patrick Reilly (TeamB)  CIS: 71333,2764
// Last:	9/18/93
// Tabs:	4

#ifndef DEF_TMACRO_H
#define DEF_TMACRO_H

// STREAMABLE()
//
// Description:
//		Inserts the appropriate method/member declarations to make a class
//	streamable. Example:
//
//	class TClass : public TView
//		{
//		public:
//			TClass(const TRect& bounds);
//
//			virtual void draw();
//			virtual TPalette& getPalette() const;
//
//		STREAMABLE(TClass)
//		};
//

#define STREAMABLE(Arg)							\
private:                                        \
	virtual const char* streamableName() const  \
		{	return name;	}                   \
protected:                                      \
	Arg(StreamableInit);                     	\
	virtual void write(opstream&);              \
	virtual void* read(ipstream&);              \
public:                                         \
	static const char* const near name;         \
	static TStreamable* build();


// STREAMABLE_NORDWR()
//
// Description:
//		Same as STREAMABLE(), except that it doesn't include the member
//	functions:
//		virtual void write(opstream&)  	and
//		virtual void* read(ipstream&).

#define STREAMABLE_NORDWR(Arg)					\
private:                                        \
	virtual const char* streamableName() const  \
		{	return name;	}                   \
protected:                                      \
	Arg(StreamableInit);                     	\
public:                                         \
	static const char* const near name;         \
	static TStreamable* build();

// STREAMABLE_COLLECTION()
//
// Description:
//		Same as STREAMABLE(), except that its for classes derived from
//	TCollection - IE read() and write() are not included, but readItem(),
//	writeItem(), and freeItem() are.

#define STREAMABLE_COLLECTION(Arg)				\
private:                                        \
	virtual const char* streamableName() const  \
		{	return name;	}                   \
protected:                                      \
	Arg(StreamableInit);                        \
	virtual void writeItem(void*, opstream&);   \
	virtual void* readItem(ipstream&);          \
	virtual void freeItem(void*);               \
public:                                         \
	static const char* const near name;         \
	static TStreamable* build();

// STREAMABLE_OPS()
//
// Description:
//		Inserts the overloaded operator<<() and operator>>() for a class.
//	Example:
//
//	class TClass : public TView
//		{...};
//
//	STREAMABLE_OPS(TClass)
//

#define STREAMABLE_OPS(Arg)                       	\
inline ipstream& operator>>(ipstream& is, Arg& cl)  \
	{	return is >> (TStreamable&)cl;	}           \
inline ipstream& operator>>(ipstream& is, Arg*& cl) \
	{	return is >> (void*&) cl;	}               \
inline opstream& operator<<(opstream& os, Arg& cl)  \
	{	return os << (TStreamable&) cl;	}           \
inline opstream& operator<<(opstream& os, Arg* cl)  \
	{	return os << (TStreamable*) cl;	}


// REGISTER_STREAMABLE()
//
// Description:
//		Inserts a streamable class' name and creates a TStremableClass.
//	Example:
//
//	#define Uses_TView
//	#define Uses_TStreamableClass    <==== need this!
//	#include <tv.h>
//	#include "TClass.h"
//
//	__link(RView)
//	REGISTER_STREAMABLE(TClass)
//
// 	Note that, unlike the TV source code, the TStreamableClass varible for
//	a class TClass is NOT (!) named RClass - its named RTClass. This means
//	if you design your modules so that you need to __link() your class, you
//	have to use __link(Rxxxx), where xxxx is the *full* class name
//	(including any leading "T").

#define REGISTER_STREAMABLE(Arg)							\
const char* const near Arg::name = #Arg;                    \
TStreamableClass R##Arg(Arg::name, Arg::build, __DELTA(Arg));

// STREAMABLE_BUILD()
//
// Description:
//		Inserts a standard builder for a streamable class. Example:
//
//	#define Uses_TView
//	#include <tv.h>
//	#include "TClass.h"
//
//	STREAMABLE_BUILD(TClass)
//

#define STREAMABLE_BUILD(Arg)                                       \
TStreamable* Arg::build()	{	return new Arg(streamableInit);	}

// STREAMABLE_CTOR()
//
// Description:
//		Inserts a standard streamable class' ctor used by the builder.
//	Example:
//
//	#define Uses_TView
//	#include <tv.h>
//	#include "TClass.h"
//
//	STREAMABLE_CTOR(TClass, TView)
//

#define STREAMABLE_CTOR(Arg, Base)                  \
Arg::Arg(StreamableInit) : Base(streamableInit)	{}

// STREAMABLE_VBCTOR()
//
// Description:
//		Just like STREAMABLE_CTOR, except is used for classes that have
//	a virtual base class. Example:
//
//	class TDlg : public TDialog
//		{...};
//
//	STREAMABLE_VBCTOR(TDlg, TDialog, TWindowInit(initFrame))
//

#define STREAMABLE_VBCTOR(Arg, Base, Virt)                  \
Arg::Arg(StreamableInit) : Base(streamableInit), Virt	{}

// STREAMABLE_CODE()
//
// Description:
//		Combines STREAMABLE_BUILD() and STREAMABLE_CTOR() into one call.
//	Example:
//
//	#define Uses_TView
//	#include <tv.h>
//	#include "TClass.h"
//
//	STREAMABLE_CODE(TClass, TView)
//

#define STREAMABLE_CODE(Arg, Base)	\
STREAMABLE_CTOR(Arg, Base)			\
STREAMABLE_BUILD(Arg)

// STREAMABLE_VBCODE()
//
// Description:
//		Just like STREAMABLE_CODE(), but for classes with virtual base class.
//	Example:
//
//	class TDlg : public TDialog
//		{...};
//
//	STREAMABLE_VBCODE(TDlg, TDialog, TWindowInit(0))
//

#define STREAMABLE_VBCODE(Arg, Base, Virt)	\
STREAMABLE_BUILD(Arg)                       \
STREAMABLE_VBCTOR(Arg, Base, Virt)

// GETPALETTE()
//
// Description:
//		Inserts the code for a standard getPalette() method. Example:
//
//	#define Uses_TView
//	#define Uses_TPalette
//	#include <tv.h>
//	#include "TClass.h"
//
//  GETPALETTE(TClass, "\x02")
//

#define GETPALETTE(Arg, PString)							\
TPalette& Arg::getPalette() const                           \
	{                                                       \
	static TPalette palette(PString, sizeof(PString)-1);    \
	return palette;                                         \
	}

#endif	// DEF_TMACRO_H
